Con  {stingray with arm/gripper/pings using remote joystick - program by Dave Ratcliff 'ratronic' 7/17/10}

  _clkmode = xtal1 + pll16x       
  _xinfreq = 5_000_000     
  
   dyna = 0            'dynamixel servo bus i/o pin
   rsrv = 8            'right ping servo output pin
   lsrv = 9            'left ping servo output pin
   rpng = 10           'right ping sensor i/o pin
   lpng = 11           'left ping sensor i/o pin
   rfrc = 17           'rf joystick receiver input pin      
   pwmb = 24           'pwm base output pin (for stingray wheel motors)          
   rspd = 80           'stingray robot scan_clear speed (up to 255max)
                                        
Obj

  dy : "dynacomv1.2"   'dynamixel servo object
  pg : "ping"          'ping object
  pw : "pwmx8"         'pulse width modulation object
  rf : "rf2"           'rf joystick receiver object
  sv : "servo32v5"     'servo object       
  
Var

  long trp1            'holds ping trigger point in inches
  long dist[2]         'holds right and left pings detected distance
  byte sf              'sonar on/off flag
  
Pub main | a, i, sp, x, y, z
  
'initialize
  dy.start(dyna, 1_000_000)                       'start dynamixel servo object on pin dyna @ 1mhz baud rate
  pw.start(pwmb, %00001111, 23_000)               'start pwm object on base pin @ 23khz
  sv.set(lsrv, 1000)                              'set pings to look straight ahead
  sv.set(rsrv, 1500)                              '        "    "
  sv.start                                        'start servo object
  rf.start(rfrc)                                  'start rf receiver object on rfrc pin
  dy.setsrvpos(1, 700)                            'set initial arm position (up)
  dy.grpposd(100)                                 'set initial dual servo gripper position (mostly open)
  trp1 := 8                                       'set initial pings trigger @ 8"
  sf := 1
  waitcnt(clkfreq*2 + cnt)                        'wait 2 seconds
  
'main loop
  repeat
  
    case rf.z                                     'set stingray operational speed
      -127..-42 : sp := 1                         'joystick full throttle
      -43..42   : sp := 2                         'joystick middle throttle
      43..127   : sp := 3                         'joystick low throttle
    x := rf.x/sp
    y := rf.y/sp
    
    sonar                                         'get pings distance                   
    if dist[0] < trp1 or dist[1] < trp1           'if either ping is less than trigger distance- 
      scan_clear                                  '-move stingray to clearest path       
         
    if rf.b == 0                                  'if no joystick button pressed- 
      stop                                        '-stop stingray moving
      
    if rf.b == 1                                  'if joystick button#1 pressed-   (trigger button)                                                
      move_robot(-y + x, -y - x)                  '-move stingray according to received joystick x,y values @ z speed
                                                  
    if rf.b == 2                                  'if joystick button#2 pressed-
      sf := 0                                     'turn sonar off
        
    if rf.b == 4                                  'if joystick button#3 pressed-
      dy.setsrvpos(1, 350)                        '-set arm position (down)
      sf := 0                                     'turn sonar off
      waitcnt(clkfreq + cnt)                      '-wait 1 second
      
    if rf.b == 8                                  'if joystick button#4 pressed-
      dy.setsrvpos(1, 600)                        '-set arm position (up)
      sf := 0                                     '-turn sonar off
      waitcnt(clkfreq*2 + cnt)                    '-wait 2 seconds
      
    if rf.b == 16                                 'if joystick button#5 pressed-
      dy.clsgrpd                                  '-close dual servo gripper         
          
    if rf.b == 32                                 'if joystick button#6 pressed-
      dy.grpposd(250)                             '-set dual servo gripper position (wide open)

    if rf.b == 64                                 'if joystick button#7 pressed-
      dy.grpposd(250)                             '-set dual servo gripper position (wide ope)   
      dy.setsrvpos(1, 350)                        '-set arm position (down)                      
      sf := 0                                     '-turn sonar off                               
      waitcnt(clkfreq + cnt)                                                                     
                       
    if rf.b == 128                                'if joystick button#8 pressed-
      dy.grpposd(225)                             '-set dual servo gripper position (wide open)
      waitcnt(clkfreq + cnt)     
      dy.setsrvpos(1, 700)                        '-set arm position (up)                           
      waitcnt(clkfreq + cnt)                      '-wait 1 second                                   
      dy.grpposd(100)                             '-set dual servo gripper position (mostly open)   
      waitcnt(clkfreq/2 + cnt)                    'wait .5 second                                   
      sf := 1                                     '-turn sonar on                                   
                   
Pub sonar | i
                                                                                           
  if not sf                                       'check sonar flag
    return                                        'return if sonar flag = 0                                                               
  repeat i from 0 to 1                            'get right and left ping measured distance in inches   
    dist[i] := pg.inches(rpng + i)                '         "       "                                    
                       
Pub scan_clear                                    'move stingray left/right towards clearest path

  if not sf                                       'check sonar flag           
    return                                        'return if sonar flag = 0   
  sv.set(lsrv, 1500)                              'set pings to look to the sides
  sv.set(rsrv, 1000)                              '        "    "
  reverse                                         'start stingray moving in reverse
  waitcnt(clkfreq/3 + cnt)                        'wait before stopping stingray
  stop                                            'stop stingray moving
  sonar                                           'get right and left pings detected distance
  sv.set(lsrv, 1000)                              'set pings to look straight ahead              
  sv.set(rsrv, 1500)                              '       "    "                                 
  if dist[0] > dist[1]                            'if distance to right is greater than distance to the left-
    right                                         '-then start moving stingray to it's right
    waitcnt(clkfreq + cnt)                        '-waste some time before stopping stingray moving       
    stop                                          '-stop stingray moving   
  else                                            'else turn stingray to the left-
    left                                          '-start stingray moving to it's left
    waitcnt(clkfreq + cnt)                        '-waste some time before stopping stingray moving     
    stop                                          '-stop stingray moving
                     
Pub stop

  move_robot(0, 0)                                'stop stingray moving
     
Pub forward

  move_robot(rspd, rspd)                          'start stingray moving forward

Pub reverse                                                                            
                                                                                              
  move_robot(-rspd, -rspd)                        'start stingray moving in reverse           
                    
Pub left

  move_robot(-rspd, rspd)                         'start stingray turning to it's left
   
Pub right 
 
  move_robot(rspd, -rspd)                         'start stingray turning to it's right
                        
Pub move_robot(left_wheel, right_wheel)           'set the speed of stingrays left and right wheels,  -255=reverse to 255=forward, 0 = stop               

  left_wheel := -255 #> left_wheel <# 255                                                                                                                                               
  right_wheel := -255 #> right_wheel <# 255                                                                                                                  
  ifnot right_wheel & $80_00_00_00             
    pw.duty(pwmb + 2, right_wheel & $ff)       
    pw.duty(pwmb + 3, 0)                      
  else                                        
    pw.duty(pwmb + 2, 0)                      
    pw.duty(pwmb + 3, (0 - right_wheel) & $ff)  
  ifnot left_wheel & $80_00_00_00                      
    pw.duty(pwmb + 1, left_wheel & $ff)          
    pw.duty(pwmb, 0)                              
  else                                            
    pw.duty(pwmb + 1, 0)                          
    pw.duty(pwmb, (0 - left_wheel) & $ff)         
                                            